package com.ruoyi.web.controller.agentcenter;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.agentcenter.domain.TAgentTask;
import com.ruoyi.agentcenter.service.ITAgentTaskService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 任务清单Controller
 * 
 * @author zhao
 * @date 2023-05-06
 */
@RestController
@RequestMapping("/agentcenter/task")
public class TAgentTaskController extends BaseController
{
    @Autowired
    private ITAgentTaskService tAgentTaskService;

    /**
     * 查询任务清单列表
     */
    @PreAuthorize("@ss.hasPermi('agentcenter:task:list')")
    @GetMapping("/list")
    public TableDataInfo list(TAgentTask tAgentTask)
    {
        startPage();
        List<TAgentTask> list = tAgentTaskService.selectTAgentTaskList(tAgentTask);
        return getDataTable(list);
    }

    /**
     * 导出任务清单列表
     */
    @PreAuthorize("@ss.hasPermi('agentcenter:task:export')")
    @Log(title = "任务清单", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TAgentTask tAgentTask)
    {
        List<TAgentTask> list = tAgentTaskService.selectTAgentTaskList(tAgentTask);
        ExcelUtil<TAgentTask> util = new ExcelUtil<TAgentTask>(TAgentTask.class);
        util.exportExcel(response, list, "任务清单数据");
    }

    /**
     * 获取任务清单详细信息
     */
    @PreAuthorize("@ss.hasPermi('agentcenter:task:query')")
    @GetMapping(value = "/get/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(tAgentTaskService.selectTAgentTaskById(id));
    }

    /**
     * 新增任务清单
     */
    @PreAuthorize("@ss.hasPermi('agentcenter:task:add')")
    @Log(title = "任务清单", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResult add(@RequestBody TAgentTask tAgentTask)
    {
        tAgentTask.setCreateBy(getUsername());
        return toAjax(tAgentTaskService.insertTAgentTask(tAgentTask));
    }

    /**
     * 修改任务清单
     */
    @PreAuthorize("@ss.hasPermi('agentcenter:task:edit')")
    @Log(title = "任务清单", businessType = BusinessType.UPDATE)
    @PutMapping(value = "/edit")
    public AjaxResult edit(@RequestBody TAgentTask tAgentTask)
    {
        tAgentTask.setUpdateBy(getUsername());
        return toAjax(tAgentTaskService.updateTAgentTask(tAgentTask));
    }

    /**
     * 删除任务清单
     */
    @PreAuthorize("@ss.hasPermi('agentcenter:task:remove')")
    @Log(title = "任务清单", businessType = BusinessType.DELETE)
	@DeleteMapping(value = "/remove/{id}")
    public AjaxResult remove(@PathVariable Long id)
    {
        return toAjax(tAgentTaskService.deleteTAgentTaskById(id));
    }
}
