package com.ruoyi.agentcenter.service.impl;

import java.util.List;
import java.util.function.Consumer;

import cn.hutool.core.lang.Assert;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.DeptUtils;
import com.ruoyi.system.mapper.SysDeptMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.apache.commons.collections4.ListUtils;
import com.ruoyi.agentcenter.mapper.TAgentTaskMapper;
import com.ruoyi.agentcenter.domain.TAgentTask;
import com.ruoyi.agentcenter.service.ITAgentTaskService;

/**
 * 任务清单Service业务层处理
 * 
 * @author zhao
 * @date 2023-07-14
 */
@Service
public class TAgentTaskServiceImpl implements ITAgentTaskService 
{
    @Autowired
    private TAgentTaskMapper tAgentTaskMapper;
    @Autowired
    private SysDeptMapper    sysDeptMapper;

    /**
     * 查询任务清单
     * 
     * @param id 任务清单主键
     * @return 任务清单
     */
    @Override
    public TAgentTask selectTAgentTaskById(Long id)
    {
        return tAgentTaskMapper.selectTAgentTaskById(id);
    }

    /**
     * 查询任务清单列表
     * 
     * @param tAgentTask 任务清单
     * @return 任务清单
     */
    @Override
    public List<TAgentTask> selectTAgentTaskList(TAgentTask tAgentTask)
    {
        return tAgentTaskMapper.selectTAgentTaskList(tAgentTask);
    }

    /**
     * 新增任务清单
     * 
     * @param tAgentTask 任务清单
     * @return 结果
     */
    @Override
    public int insertTAgentTask(TAgentTask tAgentTask)
    {
        tAgentTask.setCreateTime(DateUtils.getNowDate());
        return tAgentTaskMapper.insertTAgentTask(tAgentTask);
    }

    /**
     * 批量新增任务清单
     *
     * @param list 任务清单
     * @return 结果
     */
    @Override
    @Transactional
    public int insertTAgentTaskBatch(List<TAgentTask> list){
        List<List<TAgentTask>> splists = ListUtils.partition(list, 50);
        splists.forEach(splist->{
            tAgentTaskMapper.insertTAgentTaskBatch(splist);
        });
        return 1;
    }

    /**
     * 修改任务清单
     * 
     * @param tAgentTask 任务清单
     * @return 结果
     */
    @Override
    public int updateTAgentTask(TAgentTask tAgentTask)
    {
        tAgentTask.setUpdateTime(DateUtils.getNowDate());
        return tAgentTaskMapper.updateTAgentTask(tAgentTask);
    }

    /**
     * 批量修改 任务清单
     *
     * @param list 任务清单
     * @return 结果
     */
    @Override
    @Transactional
    public int updateTAgentTaskBatch(List<TAgentTask> list) {
        List<List<TAgentTask>> splists = ListUtils.partition(list, 50);
        splists.forEach(splist->{
            tAgentTaskMapper.updateTAgentTaskBatch(splist);
        });
        return 1;
    }

    /**
     * 批量删除任务清单
     * 
     * @param ids 需要删除的任务清单主键
     * @return 结果
     */
    @Override
    public int deleteTAgentTaskByIds(Long[] ids)
    {
        return tAgentTaskMapper.deleteTAgentTaskByIds(ids);
    }

    /**
     * 删除任务清单信息
     * 
     * @param id 任务清单主键
     * @return 结果
     */
    @Override
    public int deleteTAgentTaskById(Long id)
    {
        return tAgentTaskMapper.deleteTAgentTaskById(id);
    }

    // Harm
    /**
     * 单条条件查询任务清单
     *
     * @param tAgentTask 任务清单
     * @return 任务清单条目
     */
    @Override
    public TAgentTask selectTAgentTask(TAgentTask tAgentTask) {
        return tAgentTaskMapper.selectTAgentTask(tAgentTask);
    }

    /**
     * 条件查询任务清单数量
     *
     * @param tAgentTask 任务清单
     * @return 任务清单数量
     */
    @Override
    public Long selectTAgentTaskCount(TAgentTask tAgentTask) {
        return tAgentTaskMapper.selectTAgentTaskCount(tAgentTask);
    }

    /**
     * 条件查询任务清单是否存在
     *
     * @param tAgentTask 任务清单
     * @return 任务清单是否存在
     */
    @Override
    public boolean selectTAgentTaskExists(TAgentTask tAgentTask) {
        return tAgentTaskMapper.selectTAgentTaskExists(tAgentTask) > 0;
    }

    @Override
    public int updateTAgentTaskCount(TAgentTask tAgentTask)
    {
        return tAgentTaskMapper.updateTAgentTaskCount(tAgentTask);
    }

    @Override
    public void fillTAgentTaskOrg(TAgentTask tAgentTask)
    {
        String orgCode = tAgentTask.getOrgCode();
        int level = DeptUtils.parseOrgCodeLevel(orgCode);
        Assert.isTrue(level >= DeptUtils.LEVEL_TOWN, "orgCode必须是镇级或镇以下级别");

        SysDept deptCond = new SysDept();
        deptCond.setOrgCode(orgCode);
        SysDept org = sysDeptMapper.selectSysDept(deptCond);
        Assert.notNull(org, "orgCode对应的地区不存在: {} - {}", orgCode, level);
        List<Long> levelIds = org.makeAncestorsIdList(true);

        SysDept town;
        SysDept county;

        if(level > DeptUtils.LEVEL_TOWN) // 镇级以下
        {
            town = sysDeptMapper.selectDeptById(levelIds.get(DeptUtils.LEVEL_TOWN));
            Assert.notNull(town, "orgCode对应的镇级地区不存在: {} - {}", orgCode, level);
        }
        else
            town = org;

        county = sysDeptMapper.selectDeptById(levelIds.get(DeptUtils.LEVEL_COUNTY));
        Assert.notNull(county, "orgCode对应的区级地区不存在: {} - {}", orgCode, level);

        tAgentTask.setCountyCode(county.getOrgCode())
                .setCountyName(county.getDeptName())
                .setTownCode(town.getOrgCode())
                .setTownName(town.getDeptName())
                .setOrgName(org.getDeptName())
                  ;
    }

    @Override
    public TAgentTask getTAgentTask(String orgCode, String year, String month, Consumer<TAgentTask> ifNotExists)
    {
        TAgentTask tAgentTask = getTAgentTask(orgCode, year, month);
        if(null == tAgentTask)
        {
            tAgentTask = new TAgentTask();
            SysDept deptCond = new SysDept();
            deptCond.setOrgCode(orgCode);
            SysDept dept = sysDeptMapper.selectSysDept(deptCond);
            Assert.notNull(dept, "部门不存在: {}", orgCode);
            tAgentTask.setOrgCode(orgCode)
                .setOrderYear(year)
                .setOrderMonth(month)

                .setContracCount(0)
                .setVoucherCount(0)
                .setAssetCount(0)
                .setAllCount(0)
                .setFinishCount(0)

                .setAgentCenter(dept.getAgentCenter())
                .setAgentStatus(TAgentTask.AGENT_STATUS_READY)
            ;
            fillTAgentTaskOrg(tAgentTask);
            tAgentTask.setCreateBy("admin");
            ifNotExists.accept(tAgentTask); // 补充
            insertTAgentTask(tAgentTask);
        }
        return tAgentTask;
    }

    @Override
    public TAgentTask getTAgentTask(String orgCode, String year, String month)
    {
        Assert.notBlank(orgCode, "查询任务: orgCode为空");
        Assert.notBlank(year, "查询任务: orderYear为空");
        Assert.notBlank(month, "查询任务: orderMonth为空");

        TAgentTask task = new TAgentTask();
        task.setOrgCode(orgCode)
            .setOrderYear(year)
            .setOrderMonth(month)
        ;
        return tAgentTaskMapper.selectTAgentTask(task);
    }

    @Override
    public int syncTAgentTaskCount(TAgentTask queryTask)
    {
        TAgentTask dbTask = getTAgentTask(queryTask.getOrgCode(), queryTask.getOrderYear(), queryTask.getOrderMonth());
        Assert.notNull(dbTask, "{}下{}年{}月无任务", queryTask.getOrgCode(), queryTask.getOrderYear(), queryTask.getOrderMonth());

        TAgentTask updateTask = new TAgentTask();
        updateTask.setId(dbTask.getId());
        updateTask.setFinishCount(queryTask.getFinishCount())
                .setContracCount(queryTask.getContracCount())
                .setAssetCount(queryTask.getAssetCount())
                .setVoucherCount(queryTask.getVoucherCount())
        ;
        return updateTAgentTaskCount(updateTask);
    }
}
